/*
 Copyright (c) 2007 Kathryn Huxtable

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.

 $Id: FilterTest.java 1248 2007-04-18 12:00:08Z kathryn5 $
 */
package org.kathrynhuxtable.middleware.shibshim.filter;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Vector;

import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import junit.framework.TestCase;

import org.easymock.MockControl;
import org.kathrynhuxtable.middleware.shibshim.util.properties.PropertiesBundle;

/**
 * Test the ShibShimFilter and FilterHandler classes.
 */
public class FilterTest extends TestCase {
    
    private String assertion =
        "I1SQvfOlfXHq34pa0WBXXiIV/ulj7j+i8WSlDe1xsNu5k1fr3+tm9x9kbDtVi8U+CVfK3F0ZfRkf\n"
        + "lcPs7EZ8Dg7SqTK5ETBRGNDsdLR5JuPljfD5RAbb07qSAbMdRLshPZrK2vydJlsvg1Bg2qMOy2Y0\n"
        + "d74/R4X7JIGi/2cuuQGaBPcTnKm2t2y8jdCjBEiIoz0XemTc+PJq3g/NBw5/ed84hedS1ZB3s7xq\n"
        + "GqC4N6TfQ0kgknO3b5/KabyD00ooj8Nj0aKj5eKKi34ono075MgoadCOJLkin6dVtb+JKpOukQv2\n"
        + "jUO3wZvIEl+OZEzXuHhdeHjzpFhD2ZFLd6lP9MkD48vFKUkjkPDGCDr8Z+8L50c1r4p85UgGz3rs\n"
        + "TT9FyJnOHak9ivovhou/BNzdSJ0w8hazBc6n9sXQLa1BZxWJ6lFlhIT+CV+UyNq8+YPzU9ughCq3\n"
        + "Z2/KQgiM7GBef8EvQdNCNzpHjwoVRFXIBm49Hasy25j7wQvxlcGyQkgOH89G5JlJNumHhjto0eW+\n"
        + "IpgEBRUpLmmWnqMDQS1WyundZMVQS3YsDo+TY+Vds+BZ0zzbyV9B+QXZdW1C5wBeylemzx73Jtfs\n"
        + "Vtozj5pA7F8gp72hdIRVEG+cToA8EXfp76MG3t23h/c1CG/y1F1WMYV2pdWCAiCtw9Jodm2CynxY\n"
        + "IbiCWFGC3JrNHSOTqhXn1tyDFF3ah+/mGg388O08HMiZzh2pPYr6L4aLvwTc3UjN55hs9NRxBv+x\n"
        + "BBYLCCtcPLHy/duHKy7wMtOocfwTyAA2Rdje2J7X2gfwYRO5mKnYalrL40YGK39w7I1bikDduBOJ\n"
        + "1Co+MC/9hWulCP79ozzbINfTTgJ9Gj08rHVqjeJZ6yU76qO3lA1v4r6ONpusLrn5fG1UKg9vZy68\n"
        + "kTWmcLRlBzLEg614WlV8qbsv2wrW3IMUXdqH7+YaDfzw7TwcclOAvKnWStPJAwOh1Yppa9w/KtVL\n"
        + "OCC66lQSL7cRRVzUnx8WAEi3Xr/lnnFgJ4FNU0dqBeHddt77HFEHjFcw9A==\n";
    private String signature =
        "1xmV5At3qn3dGYU6yc2KWB0rivxJ+Xxx1vJNu8jQ2kI8uJeboU38akBG2M7Cpk745J+c0Ff8oqZ0\n"
        + "H3KNdgfulcPXCWTNTUvWjSJPrSaSCPRTJp8ZD5trG03kBeqbBijTRT3zy6+lb9CFU8iXhVFqNLVR\n"
        + "4eApXn7qia0ghmpcaC+JoNiFeGRI7JFHfAEVryMeVzE+7AJru5ZLZQFqTclLn3BCQSlf4ly1ceoM\n"
        + "Pa89rOATlinJW/U5SfLpgsA7HRBa4iP8o0PVXPOGavvIDuA5zs8kiEmpQpBdxEasc48ljrbqD/5L\n"
        + "eK++YAXXtl+2UZ+V72awdcWVHZQRqzpb3esSqw==\n";
    private String assertionText =
        "<Assertion IssueInstant=\"2007-04-15T01:36:43.471Z\" AuthenticationInstant=\"2007-04-15T01:36:43.439Z\""
        + " Issuer=\"https://idp.example.edu/shibboleth-idp/SSO\" Handle=\"0123456789ABCDEF\">"
        + "<AttributeStatement>"
        + "<Attribute AttributeName=\"shib-em-ismemberof\">"
        + "<AttributeValue>example:group1</AttributeValue>"
        + "<AttributeValue>example:group2</AttributeValue>"
        + "<AttributeValue>example:group3</AttributeValue>"
        + "</Attribute>"
        + "<Attribute AttributeName=\"shib-ep-affiliation\">"
        + "<AttributeValue>staff@example.edu</AttributeValue>"
        + "<AttributeValue>employee@example.edu</AttributeValue>"
        + "<AttributeValue>member@example.edu</AttributeValue>"
        + "</Attribute>"
        + "<Attribute AttributeName=\"shib-ep-edupersonprincipalname\">"
        + "<AttributeValue>myusername@example.edu</AttributeValue>"
        + "</Attribute>"
        + "</AttributeStatement>"
        + "</Assertion>";

    private String cryptKey = "G4kslxK4GZdpD+wufCxnTQ==\n";
    private String serverCert =
        "-----BEGIN CERTIFICATE-----\n"
        + "MIIEVDCCAzygAwIBAgIJAP24RcTW1aetMA0GCSqGSIb3DQEBBAUAMHkxCzAJBgNV\n"
        + "BAYTAlVTMRAwDgYDVQQIEwdFeGFtcGxlMRMwEQYDVQQHEwpFeGFtcGxldG9uMRsw\n"
        + "GQYDVQQKExJFeGFtcGxlIFVuaXZlcnNpdHkxCzAJBgNVBAsTAklUMRkwFwYDVQQD\n"
        + "ExBUZXN0IENlcnRpZmljYXRlMB4XDTA3MDMyOTE2NTMyNFoXDTE3MDMyNjE2NTMy\n"
        + "NFoweTELMAkGA1UEBhMCVVMxEDAOBgNVBAgTB0V4YW1wbGUxEzARBgNVBAcTCkV4\n"
        + "YW1wbGV0b24xGzAZBgNVBAoTEkV4YW1wbGUgVW5pdmVyc2l0eTELMAkGA1UECxMC\n"
        + "SVQxGTAXBgNVBAMTEFRlc3QgQ2VydGlmaWNhdGUwggEiMA0GCSqGSIb3DQEBAQUA\n"
        + "A4IBDwAwggEKAoIBAQDu4RMs8dbl7UDhbk9OclUpTz2ygT06A05w81Iv7lWRFSQS\n"
        + "g+PBRZPdOFNu8kJfP5rveibfdvQZMxysTzESno3y2yrRGDCm9aQiunx/tvDJvXrA\n"
        + "dxaJ2pUKZXAklh3KVY5fXAJgTFKzODmKtk5x7L4d4UB0gV+/d5fVLz21JVWHaN9Q\n"
        + "05ngdORgjVZYd3vw7p8ZT4IJc91niZMXRy2w4KnLSNP/qa3bVr8QAGRQTRsN0j0m\n"
        + "FnqKFMbLgL2asbOeH6AyF8FhvDIPOVW1LzkZQy9Gf2b1NlcymGMV9twjOMH+07tA\n"
        + "m7DMiGTon5AJIzVY/CEvlah8qYi602Q8irw5hMl/AgMBAAGjgd4wgdswHQYDVR0O\n"
        + "BBYEFHvsCpxRonD+WKqVTEg2+2Oy4dKGMIGrBgNVHSMEgaMwgaCAFHvsCpxRonD+\n"
        + "WKqVTEg2+2Oy4dKGoX2kezB5MQswCQYDVQQGEwJVUzEQMA4GA1UECBMHRXhhbXBs\n"
        + "ZTETMBEGA1UEBxMKRXhhbXBsZXRvbjEbMBkGA1UEChMSRXhhbXBsZSBVbml2ZXJz\n"
        + "aXR5MQswCQYDVQQLEwJJVDEZMBcGA1UEAxMQVGVzdCBDZXJ0aWZpY2F0ZYIJAP24\n"
        + "RcTW1aetMAwGA1UdEwQFMAMBAf8wDQYJKoZIhvcNAQEEBQADggEBAJ65MBZvvMjx\n"
        + "RUDS4a4EgcTbAxCd8Q4giAmKAGUMo9C3cGe+cjisiLInXp8wR1FEj44rjKLvKlKD\n"
        + "q5BgNUSMtqxj65zLOrVD9OjVa8qbbmp8cNQxSYROw2wtvgJRbBlZ85MDcczFYe4N\n"
        + "Nrst578Q77RCZCk+enEpniLi3kc0ZMMDhqLDGBUXwCswu7kWaiv+L6oqk4PWWNbh\n"
        + "Jqf9Ckf4MSzhcFILRkAxDJszvLUmQ/JWvNFfUCmCPl8dM+lT8g9h86ykqf9fVyES\n"
        + "loIIxg13uLOevDG33SCUsIUebmVSAUQKnw3xchNgtrSm0CN6P3/pzIoYdwZYN9XH\n"
        + "BV7se9Ch35I=\n"
        + "-----END CERTIFICATE-----\n";

    private MockControl         controlConfig;
    private FilterConfig        config;
    private MockControl         controlRequest;
    private HttpServletRequest  request;
    private MockControl         controlResponse;
    private HttpServletResponse response;
    private MockControl         controlSession;
    private HttpSession         session;
    private MyChain             chain;
    private ShibShimFilter      filter;

    public void setUp() {
        controlConfig = MockControl.createControl(FilterConfig.class);
        config = (FilterConfig) controlConfig.getMock();
        controlRequest = MockControl.createControl(HttpServletRequest.class);
        request = (HttpServletRequest) controlRequest.getMock();
        controlResponse = MockControl.createControl(HttpServletResponse.class);
        response = (HttpServletResponse) controlResponse.getMock();
        controlSession = MockControl.createControl(HttpSession.class);
        session = (HttpSession) controlSession.getMock();
        
        chain = new MyChain();

        filter = new ShibShimFilter();
    }

    public void tearDown() {
        controlConfig.verify();
        controlRequest.verify();
        controlResponse.verify();
        controlSession.verify();

        PropertiesBundle.clearForTesting();
    }

    public void testInitFailure1() {
        config.getInitParameter("ShibShimFilterPropertiesFile");
        controlConfig.setReturnValue(null);

        controlConfig.replay();
        controlRequest.replay();
        controlResponse.replay();
        controlSession.replay();

        try {
            filter.init(config);
        } catch (ServletException e) {
            assertTrue(true);
            return;
        }
        fail("Failed to catch ServletException calling filter.init: ");
    }

    public void testInitFailure2() {
        config.getInitParameter("ShibShimFilterPropertiesFile");
        controlConfig.setReturnValue("/unreadable.properties");

        controlConfig.replay();
        controlRequest.replay();
        controlResponse.replay();
        controlSession.replay();

        try {
            filter.init(config);
        } catch (ServletException e) {
            assertTrue(true);
            return;
        }
        fail("Failed to catch ServletException calling filter.init: ");
    }

    public void testInitFailure3() {
        config.getInitParameter("ShibShimFilterPropertiesFile");
        controlConfig.setReturnValue("/nonexistent.properties");

        controlConfig.replay();
        controlRequest.replay();
        controlResponse.replay();
        controlSession.replay();

        try {
            filter.init(config);
        } catch (ServletException e) {
            assertTrue(true);
            return;
        }
        fail("Failed to catch ServletException calling filter.init: ");
    }

    public void testInit() {
        config.getInitParameter("ShibShimFilterPropertiesFile");
        controlConfig.setReturnValue("/test.properties");

        controlConfig.replay();
        controlRequest.replay();
        controlResponse.replay();
        controlSession.replay();

        try {
            filter.init(config);
        } catch (ServletException e) {
            fail("Got ServletException calling filter.init: " + e);
        }

        assertEquals("shib-edupersonprincipalname", filter.getRemoteUserAttribute());
        assertEquals(cryptKey, filter.getShibShimServerCryptKey());
        assertEquals(serverCert, filter.getShibShimServerCert());
        assertEquals(120000, filter.getTimeout());
        
        filter.destroy();
    }

    public void testBadRequestClass() {
        config.getInitParameter("ShibShimFilterPropertiesFile");
        controlConfig.setReturnValue("/test.properties");
        try {
            response.getWriter();
        } catch (IOException e) {
            // This shouldn't happen.
            fail("Got IOException calling mock response.getWriter(): " + e);
        }
        controlResponse.setThrowable(new IOException("Foo"));

        controlConfig.replay();
        controlRequest.replay();
        controlResponse.replay();
        controlSession.replay();

        try {
            filter.init(config);
        } catch (ServletException e) {
            fail("Got ServletException calling filter.init: " + e);
        }
        filter.doFilter(null, response, chain);
        assertTrue(true);
    }

    public void testInitialRedirect() {
        config.getInitParameter("ShibShimFilterPropertiesFile");
        controlConfig.setReturnValue("/test.properties");

        request.getServletPath();
        controlRequest.setReturnValue("/mypath", 4);
        request.getServerName();
        controlRequest.setReturnValue("myapp.example.org");
        request.getServerPort();
        controlRequest.setReturnValue(443);
        request.getContextPath();
        controlRequest.setReturnValue("/myapp", 3);
        request.getSession(false);
        controlRequest.setReturnValue(null);
        request.getSession(true);
        controlRequest.setReturnValue(session);
        session.setAttribute("org.kathrynhuxtable.middleware.shibshim.userAttributes", null);
        controlSession.setMatcher(MockControl.ALWAYS_MATCHER);
        request.getRequestURL();
        controlRequest.setReturnValue(new StringBuffer("https://myapp.example.org/myapp/mypath"));
        request.getQueryString();
        controlRequest.setReturnValue(null);
        try {
            response.sendRedirect("https://myshim.example.edu/shibshim/ACS/https/myapp.example.org/myapp/localACS?target=%2Fmypath");
        } catch (IOException e) {
            // This won't happen. This is a mock object.
            fail("Got IOException calling mock response.sendRedirect: " + e);
        }

        controlConfig.replay();
        controlRequest.replay();
        controlResponse.replay();
        controlSession.replay();

        try {
            filter.init(config);
        } catch (ServletException e) {
            fail("Got ServletException calling filter.init: " + e);
        }
        filter.doFilter(request, response, chain);
        assertTrue(true);
    }

    public void testInitialRedirectQueryString() {
        config.getInitParameter("ShibShimFilterPropertiesFile");
        controlConfig.setReturnValue("/test.properties");

        request.getServletPath();
        controlRequest.setReturnValue("/mypath", 4);
        request.getServerName();
        controlRequest.setReturnValue("myapp.example.org");
        request.getServerPort();
        controlRequest.setReturnValue(443);
        request.getContextPath();
        controlRequest.setReturnValue("/myapp", 3);
        request.getSession(false);
        controlRequest.setReturnValue(null);
        request.getSession(true);
        controlRequest.setReturnValue(session);
        session.setAttribute("org.kathrynhuxtable.middleware.shibshim.userAttributes", null);
        controlSession.setMatcher(MockControl.ALWAYS_MATCHER);
        request.getRequestURL();
        controlRequest.setReturnValue(new StringBuffer("https://myapp.example.org/myapp/mypath"));
        request.getQueryString();
        controlRequest.setReturnValue("foo=bar");
        try {
            response.sendRedirect("https://myshim.example.edu/shibshim/ACS/https/myapp.example.org/myapp/localACS?target=%2Fmypath%3Ffoo%3Dbar");
        } catch (IOException e) {
            // This won't happen. This is a mock object.
            fail("Got IOException calling mock response.sendRedirect: " + e);
        }

        controlConfig.replay();
        controlRequest.replay();
        controlResponse.replay();
        controlSession.replay();

        try {
            filter.init(config);
        } catch (ServletException e) {
            fail("Got ServletException calling filter.init: " + e);
        }
        filter.doFilter(request, response, chain);
        assertTrue(true);
    }

    public void testInitialRedirectDifferentPort() {
        config.getInitParameter("ShibShimFilterPropertiesFile");
        controlConfig.setReturnValue("/test.properties");

        request.getServletPath();
        controlRequest.setReturnValue("/mypath", 4);
        request.getServerName();
        controlRequest.setReturnValue("myapp.example.org");
        request.getServerPort();
        controlRequest.setReturnValue(8443, 2);
        request.getContextPath();
        controlRequest.setReturnValue("/myapp", 3);
        request.getSession(false);
        controlRequest.setReturnValue(null);
        request.getSession(true);
        controlRequest.setReturnValue(session);
        session.setAttribute("org.kathrynhuxtable.middleware.shibshim.userAttributes", null);
        controlSession.setMatcher(MockControl.ALWAYS_MATCHER);
        request.getRequestURL();
        controlRequest.setReturnValue(new StringBuffer("https://myapp.example.org:8443/myapp/mypath"));
        request.getQueryString();
        controlRequest.setReturnValue(null);
        try {
            response.sendRedirect("https://myshim.example.edu/shibshim/ACS/https/myapp.example.org:8443/myapp/localACS?target=%2Fmypath");
        } catch (IOException e) {
            // This won't happen. This is a mock object.
            fail("Got IOException calling mock response.sendRedirect: " + e);
        }

        controlConfig.replay();
        controlRequest.replay();
        controlResponse.replay();
        controlSession.replay();

        try {
            filter.init(config);
        } catch (ServletException e) {
            fail("Got ServletException calling filter.init: " + e);
        }
        filter.doFilter(request, response, chain);
        assertTrue(true);
    }

    public void testACSRedirect() {
        config.getInitParameter("ShibShimFilterPropertiesFile");
        controlConfig.setReturnValue("/test.properties");

        request.getServletPath();
        controlRequest.setReturnValue("/localACS", 5);
        request.getServerName();
        controlRequest.setReturnValue("myapp.example.org");
        request.getServerPort();
        controlRequest.setReturnValue(443);
        request.getContextPath();
        controlRequest.setReturnValue("/myapp", 2);
        request.getSession(false);
        controlRequest.setReturnValue(session);
        session.getAttribute("org.kathrynhuxtable.middleware.shibshim.userAttributes");
        controlSession.setReturnValue(null);
        session.setAttribute("org.kathrynhuxtable.middleware.shibshim.userAttributes", null);
        controlSession.setMatcher(MockControl.ALWAYS_MATCHER);
        request.getParameter("assertion");
        controlRequest.setReturnValue(assertion);
        request.getParameter("signature");
        controlRequest.setReturnValue(signature);
        request.getParameter("target");
        controlRequest.setReturnValue("/mypath");
        try {
            response.sendRedirect("/myapp/mypath");
        } catch (IOException e) {
            // This won't happen. This is a mock object.
            fail("Got IOException calling mock response.sendRedirect: " + e);
        }

        controlConfig.replay();
        controlRequest.replay();
        controlResponse.replay();
        controlSession.replay();

        try {
            filter.init(config);
        } catch (ServletException e) {
            fail("Got ServletException calling filter.init: " + e);
        }
        filter.doFilter(request, response, chain);
        assertTrue(true);
    }

    public void testFilterWrapper() {
        HashMap attributeMap = new HashMap();
        attributeMap.put("shib-em-ismemberof", "shib-ismemberof");
        attributeMap.put("shib-ep-affiliation", "shib-affiliation");
        attributeMap.put("shib-ep-edupersonprincipalname", "shib-edupersonprincipalname");
        UserAttributesImpl attributes = new UserAttributesImpl(attributeMap);
        try {
            attributes.parseUserInfo(assertionText);
        } catch (UserInfoException e) {
            fail("Got UserInfoException creating sample UserInfo");
        }
        
        Vector v = new Vector();
        v.add("foo");
        
        Vector v1 = new Vector();
        v1.add("bar");

        config.getInitParameter("ShibShimFilterPropertiesFile");
        controlConfig.setReturnValue("/test.properties");

        request.getServletPath();
        controlRequest.setReturnValue("/mypath", 5);
        request.getServerName();
        controlRequest.setReturnValue("myapp.example.org");
        request.getServerPort();
        controlRequest.setReturnValue(443);
        request.getContextPath();
        controlRequest.setReturnValue("/myapp", 1);
        request.getSession(false);
        controlRequest.setReturnValue(session);
        session.getAttribute("org.kathrynhuxtable.middleware.shibshim.userAttributes");
        controlSession.setReturnValue(attributes);
        request.getHeaderNames();
        controlRequest.setReturnValue(v.elements());
        request.getHeaders("foo");
        controlRequest.setReturnValue(v1.elements());
        request.getHeader("foo");
        controlRequest.setReturnValue("bar");

        controlConfig.replay();
        controlRequest.replay();
        controlResponse.replay();
        controlSession.replay();

        try {
            filter.init(config);
        } catch (ServletException e) {
            fail("Got ServletException calling filter.init: " + e);
        }
        filter.doFilter(request, response, chain);
        
        Enumeration e = chain.getRequest().getHeaderNames();
        ArrayList list = new ArrayList();
        while (e.hasMoreElements()) {
            list.add(e.nextElement());
        }
        assertEquals(4, list.size());
        assertTrue(list.contains("foo"));
        assertTrue(list.contains("shib-affiliation"));
        assertTrue(list.contains("shib-edupersonprincipalname"));
        assertTrue(list.contains("shib-ismemberof"));

        assertEquals("myusername@example.edu", chain.getRequest().getRemoteUser());

        e = chain.getRequest().getHeaders("shib-affiliation");
        list = new ArrayList();
        while (e.hasMoreElements()) {
            list.add(e.nextElement());
        }
        assertEquals(3, list.size());
        assertTrue(list.contains("staff@example.edu"));
        assertTrue(list.contains("employee@example.edu"));
        assertTrue(list.contains("member@example.edu"));
        
        e = chain.getRequest().getHeaders("foo");
        list = new ArrayList();
        while (e.hasMoreElements()) {
            list.add(e.nextElement());
        }
        assertEquals(1, list.size());
        assertTrue(list.contains("bar"));

        assertEquals("bar", chain.getRequest().getHeader("foo"));
    }

    public void testFilterWrapperNoRemoteUser() {
        HashMap attributeMap = new HashMap();
        attributeMap.put("shib-em-ismemberof", "shib-ismemberof");
        attributeMap.put("shib-ep-affiliation", "shib-affiliation");
        attributeMap.put("shib-ep-edupersonprincipalname", "shib-edupersonprincipalname");
        UserAttributesImpl attributes = new UserAttributesImpl(attributeMap);
        try {
            attributes.parseUserInfo(assertionText);
        } catch (UserInfoException e) {
            fail("Got UserInfoException creating sample UserInfo");
        }
        
        Vector v = new Vector();
        v.add("foo");
        
        Vector v1 = new Vector();
        v1.add("bar");

        config.getInitParameter("ShibShimFilterPropertiesFile");
        controlConfig.setReturnValue("/test-no-remote-user.properties");

        request.getServletPath();
        controlRequest.setReturnValue("/mypath", 5);
        request.getServerName();
        controlRequest.setReturnValue("myapp.example.org");
        request.getServerPort();
        controlRequest.setReturnValue(443);
        request.getContextPath();
        controlRequest.setReturnValue("/myapp", 1);
        request.getSession(false);
        controlRequest.setReturnValue(session);
        session.getAttribute("org.kathrynhuxtable.middleware.shibshim.userAttributes");
        controlSession.setReturnValue(attributes);
        request.getHeaderNames();
        controlRequest.setReturnValue(v.elements());
        request.getRemoteUser();
        controlRequest.setReturnValue("remote@somedomain");

        controlConfig.replay();
        controlRequest.replay();
        controlResponse.replay();
        controlSession.replay();

        try {
            filter.init(config);
        } catch (ServletException e) {
            fail("Got ServletException calling filter.init: " + e);
        }
        filter.doFilter(request, response, chain);
        
        Enumeration e = chain.getRequest().getHeaderNames();
        ArrayList list = new ArrayList();
        while (e.hasMoreElements()) {
            list.add(e.nextElement());
        }
        assertEquals(4, list.size());
        assertTrue(list.contains("foo"));
        assertTrue(list.contains("shib-affiliation"));
        assertTrue(list.contains("shib-edupersonprincipalname"));
        assertTrue(list.contains("shib-ismemberof"));

        assertEquals("remote@somedomain", chain.getRequest().getRemoteUser());
    }

    /**
     * Stub class for FilterChain. We need this to retrieve the values passed to
     * it in the request wrapper.
     */
    public class MyChain implements FilterChain {
        private HttpServletRequest  request  = null;
        private HttpServletResponse response = null;

        /**
         * @return the servlet wrapper request.
         */
        public HttpServletRequest getRequest() {
            return request;
        }

        /**
         * @return the original response object.
         */
        public HttpServletResponse getResponse() {
            return response;
        }

        /**
         * {@inheritDoc}
         */
        public void doFilter(ServletRequest request, ServletResponse response) {
            this.request = (HttpServletRequest) request;
            this.response = (HttpServletResponse) response;
        }
    }
}
